@@ -1,10 +1,9 @@
 """HydroMT: Build and analyze models like a data-wizard!"""
 
-__version__ = "0.4.0"
+__version__ = "v0.4.2"
 
 import geopandas as gpd
-from os.path import join, isdir, dirname, basename, isfile, abspath
-import glob
+import warnings
 
 # required for accessor style documentation
 from xarray import DataArray, Dataset
@@ -31,3 +30,15 @@ from . import cli, workflows, stats, flw, raster, vector
 from .models import *
 from .io import *
 from .data_adapter import *
+
+# TODO remove after fixed in all plugins
+def __getattr__(name):
+    if name in PLUGINS:
+        ep = ENTRYPOINTS[PLUGINS[name]]
+        plugin_name = ep.module_name.split(".")[0]
+        warnings.warn(
+            f'"hydromt.{name}" will be deprecated, use "{plugin_name}.{name}" instead.',
+            DeprecationWarning,
+        )
+        return model_plugins.load(ep)
+    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")
